<!--- 
	csw-map-markers-basic-google-genericRH.cfm
	
	This 'generic' render handler render handler implements the 'getGenericFieldMappingsArray' function below to describe
	the available field that can be mapped.
	
	The logic above the function, renders an image with a caption above it, and optionally renders a description 
	and link.
	
	Note that for 'generic' render handlers the mapped data, if any, will always be passed in the following structure:
		Attributes.ElementInfo.ElementData.PropertyValues
	This is an array of one or more items, with a sub-structure named 'values'.  
	Values is a structure with members as defined by the getGenericFieldMappingsArray() function.
	
	This generic render handler assumes these fields:
		- ID - hidden (gets a default value)
		- Name - Required
		- Latitude (floating point number (positive or negative))
		- Longitude (floating point number (positive or negative))
		- Title
		- Zoom  (integer)
		- Markers  (custom element data manager)
		
--->

<cfscript>
	// TO CACHE THIS RH - comment,  remove or set isStatic=1
	//request.element.isStatic = 0;
	
	application.ADF.scripts.loadResources("JQuery");
	application.ADF.scripts.loadResources("MaplaceJS");
	application.ADF.scripts.loadResources("cswMapBasicGoogleCSS");
	/* Load Custom CSS Resource */
	Application.ADF.widgets.loadCustomWidgetCSS("cswMapBasicGoogleCSS","TERTIARY","_custom");
	
	// exit if all we're doing is detecting required resources
	if ( Request.RenderState.RenderMode EQ "getResources" )
		exit;
	
	if ( NOT structKeyExists(attributes,'elementinfo') )
		exit;
	
	// Set the element ID
	elementID = request.element.id;

	// Get Records from the ElementData structure
	elementData = Attributes.ElementInfo.ElementData;
	items = ArrayNew(1);

	// Get Records from the Element as an Array of Structures
	if ( StructKeyExists(elementData, 'PropertyValues') AND ArrayLen(elementData.PropertyValues) )
		items = elementData.PropertyValues;

	totalItems = ArrayLen(items);	

	// Get the RenderHandler MetaData
	rhMetadata = Attributes.ElementInfo.RenderHandlerMetaData;
	
	widgetData = StructNew();
	requireInitialWidgetData = true;
	
	// Set the Field Names for Render Handler Metadata to WidgetData conversion
	mdOptionsFieldName = "WidgetOptions,APIkey";
	mapAPIkey = "";
	
	/* Verify the Widget Custom Medadata Form and Render Handler Binding */
	/* Look at the ElementInfo struct and the Caller Struct verify the metadata form is bound and we have some saved data */
	mdBindingIsValid = Application.ADF.widgets.verifyRenderHandlerMetadata(rhCaller=Variables.Caller,metadataFields=mdOptionsFieldName,rhElementInfo=Attributes.ElementInfo,requireMetadata=requireInitialWidgetData);
	if ( mdBindingIsValid )
	{
		// Get the widgetData from the bound Metadata Form with a "WidgetOptions" field
		widgetData = Application.ADF.widgets.getWidgetConfigData(customMetdata=rhMetadata,configFieldName=mdOptionsFieldName);
	
		// Also, make sure we have a Google Maps API key
		if ( !StructKeyExists(widgetData,"APIkey") OR LEN(TRIM(widgetData.APIkey)) EQ 0 )
		{
			WriteOutput("<div><strong><em>Please open the Custom Properties for this Render Handler and add a Google Maps API key!</em></strong></div>");
		}
		else
			mapAPIkey = widgetData.APIkey;
	}
	
	/* PARSE [Theme] */
	themeClass = "";
	if ( StructKeyExists(widgetData,"Theme") )
		themeClass = TRIM(widgetData.Theme);
	
	/* PARSE [zoom] */
	zoomValue = "";
	if ( StructKeyExists(widgetData,"zoom") )
		zoomValue = INT(widgetData.zoom);
	
	/* PARSE [width] */
	widthStr = "";
	if ( StructKeyExists(widgetData,"width") AND LEN(TRIM(widgetData.width)) )
		widthStr = widgetData.width;
		
	/* PARSE [height] */
	heightStr = "";
	if ( StructKeyExists(widgetData,"height") AND LEN(TRIM(widgetData.height)) )
		heightStr = widgetData.height;
	
	// Open the MarkerSet
	markerSet = "";
	markerSetOpen = "var markerLocs_#elementID# = [";
	markerSetClose = "]";
	
	currTitle = "";
	currHTML = "";
	animatedState = "";
	iconURL = "";
</cfscript>

<cfif totalItems>

	<cfif !StructKeyExists(request,"cswMapMarkersBasicGoogle") OR !StructKeyExists(request.cswMapMarkersBasicGoogle,hash(mapAPIkey))>
		<cfsavecontent variable="googleMapMarkersBasicApiJS">
			<cfoutput>
				<script src="https://maps.google.com/maps/api/js?libraries=geometry&key=#widgetData.APIkey#"></script>
			</cfoutput>
		</cfsavecontent>
		<cfset request.cswMapMarkersBasicGoogle[hash(widgetData.APIkey)] = 1>
		<!---Load the Google Maps JS--->
		<cfset application.ADF.scripts.addFooterHTML(googleMapMarkersBasicApiJS, "SECONDARY")> <!---resourceGroup: PRIMARY, SECONDARY, TERTIARY--->
	</cfif>

	<cfoutput><div class="csw-Map-Basic-Google #themeClass#"></cfoutput>

		<cfset markerSet = markerSetOpen>
		<cfloop from="1" to="#totalItems#" index="itm">
			<cfscript>
				// Custom Element data coming from CS render mode filter so unescaping is needed before applying jsstringformat()
				currTitle = "";
				if ( StructKeyExists(items[itm].values,'title') AND LEN(TRIM(items[itm].values.title)) )
					currTitle = items[itm].values.title;
				currTitle = server.commonspot.udf.html.unescape(currTitle);
				currTitle = jsstringformat(currTitle);
				
				currHTML = "";
				if ( StructKeyExists(items[itm].values,'infoWindowHTML') AND LEN(TRIM(items[itm].values.infoWindowHTML)) )
					currHTML = items[itm].values.infoWindowHTML;
				currHTML = server.commonspot.udf.html.unescape(trim(currHTML));
				currHTML = jsstringformat(currHTML);

				// Build the Marker Options
				animatedState = "";
				if ( StructKeyExists(items[itm].values,'animated') AND items[itm].values.animated EQ 1 )
					animatedState = ",animation: google.maps.Animation.DROP";

				iconURL = "";
				if ( StructKeyExists(items[itm].values,'icon') AND isStruct(items[itm].values.icon) AND StructKeyExists(items[itm].values.icon,"resolvedURL") AND StructKeyExists(items[itm].values.icon.resolvedURL,"serverRelative") )
				{
					// assume CommonSpot icon field is used and structure is passed
					iconURL = items[itm].values.icon.ResolvedURL.ServerRelative;
				}
				else
				{
					// assume icon url string is passed 
					iconURL = items[itm].values.icon;
				}
				
				markerSet = markerSet & "
					{
						lat: #items[itm].values.latitude#,
						lon: #items[itm].values.longitude#,
						title: '#currTitle#',
						html: '#currHTML#',
						icon: '#iconURL#'
						#animatedState#
					}
				";

				if ( itm LT totalItems )
					markerSet = markerSet & ",";
			</cfscript>
		</cfloop>
		<cfset markerSet = markerSet & markerSetClose>

		<cfsavecontent variable="googleMapMarkersBasicRenderJS">
			<cfoutput>
				<script>
					#markerSet#

					jQuery(function () {
						new Maplace({
							map_div: '##cswGoogleMap-#elementID#',
							locations: markerLocs_#elementID#,
							generate_controls: false,
							start: 1,
							map_options: {
								zoom: #zoomValue#,
								scrollwheel: false
							}
							,listeners: {
								  tilesloaded: function(evt) {
										// Add alt tags to the Google Maps for ADA
										jQuery(this.getDiv()).find("img").each(function(i, eimg){
											if(!eimg.alt || eimg.alt ===""){
												eimg.alt = "Google Maps Image";
											}
										});
								  }
							}
						}).Load();
					});
				</script>
			</cfoutput>
		</cfsavecontent>

		<!---Load the Maplace Custom JS--->
		<cfset application.ADF.scripts.addFooterJS(googleMapMarkersBasicRenderJS, "SECONDARY")> <!---resourceGroup: PRIMARY, SECONDARY, TERTIARY--->

		<cfoutput>
		<div class="cswItems cswItem">
			<div class="cswMap" id="cswGoogleMap-#elementID#" style="width:#widthStr#;height:#heightStr#;"></div><br>
			<!--- <div class="cswRoute" id="gmap-route_#elementID#_#itm#"></div> --->
		</div>
		</cfoutput>

	<cfoutput></div></cfoutput>
</cfif>

<!--- ////////////////////////////////////////////////////////////////////////// --->
<!--- //////////////////   GENERIC RENDER HANDLER FUNCTIONS   ////////////////// --->
<!--- ////////////////////////////////////////////////////////////////////////// --->

<!---
	This is the function that is called by CommonSpot when this render handler is registred, allowing teh user to 
	map the fields of the source element to this render handler.
	
	Here we expose build a structure describing the four (4) fields that can be mapped. Two (2) of the fields are marked 
	as required. The other two are optional.
		- Title - Required
		- Description - Required
		- Image  (string or csImage structure)
		- LinkURL  ((string or csExtURL structure)
		- LinkText (string)
	
	Each fieldMappingArray item is a structure with the following 3 fields:
		- FieldName
		- Description
		- Required
--->
<cffunction name="getGenericFieldMappingsArray" returntype="array" access="private" output="no">
	<cfscript>
		var fieldMappingArray = ArrayNew(1);
		var fieldDetailsStruct = StructNew();
		
		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'id';
		fieldDetailsStruct.Description = 'Unique identifier for this marker.';
		fieldDetailsStruct.Required = 1;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);
		
		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'Name';
		fieldDetailsStruct.Description = 'The internal name for the marker.';
		fieldDetailsStruct.Required = 1;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);
		
		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'Longitude';
		fieldDetailsStruct.Description = "Starting location for this map. Specifies the east-west position of a point on the Earth's surface.";
		fieldDetailsStruct.Required = 1;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);
		
		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'Latitude';
		fieldDetailsStruct.Description = "Starting location for this map. Specifies the north-south position of a point on the Earth's surface.";
		fieldDetailsStruct.Required = 1;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);

		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'Title';
		fieldDetailsStruct.Description = 'The title for the marker.';
		fieldDetailsStruct.Required = 0;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);
		
		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'infoWindowHTML';
		fieldDetailsStruct.Description = 'An info window can be created for each marker using specified HTML.';
		fieldDetailsStruct.Required = 0;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);

		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'icon';
		fieldDetailsStruct.Description = 'Typically a pointer-type icon is used like you would see on Google Maps.';
		fieldDetailsStruct.Required = 0;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);

		fieldDetailsStruct = StructNew();
		fieldDetailsStruct.FieldName = 'animated';
		fieldDetailsStruct.Description = 'Checking this box will cause the marker to drop from the top of the map down to its location when created.';
		fieldDetailsStruct.Required = 0;
		arrayAppend(fieldMappingArray, fieldDetailsStruct);

		return fieldMappingArray;
	</cfscript>
</cffunction>